# config.py

import logging

__version__ = "0.7.1"

#: Sets application-wide loglevel
LOG_LEVEL = logging.DEBUG
#: Set logger message format
LOG_FORMAT = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
